program DrawDesignerScript;

{
ExitCodes:
  0 - o.k.
  1 - Unknown error
  2 - Output file is locked
}

function GetDC(hWnd: HWND): HDC; external 'GetDC@user32.dll stdcall';
function ReleaseDC(hWnd: HWND; hDC: HDC): Integer; external 'ReleaseDC@user32.dll stdcall';
function GetDeviceCaps(DC: HDC; Index: Integer): Integer; external 'GetDeviceCaps@gdi32.dll stdcall';
function ConvertEmfToPDFFileA(InputFile:PAnsiChar; OutputFile:PAnsiChar; Title, Subject, Author, Keywords:PAnsiChar):Longint; external 'ConvertEmfToPDFFileA@emf2pdf.dll stdcall';


type
  TDrawPaperSize = (dpsUserDefined, dpsA0, dpsA1, dpsA2, dpsA3, dpsA4, dpsA5, dpsA6, dpsA7, dpsLetter, dpsLegal, dpsJuniorLegal, dpsLedger, dpsTabloid);
  TDrawPageOrientation = (dpoPortrait, dpoLandscape);
  TDrawTargetFormat = (dtfDefault, dtfPdf, dtfMultiEmf);

  TCommonOptions = record
    OptionsFile:string;
    TargetFile:string;
    AlwaysSaveEMF:boolean;
    DocAuthor:string;
    DocKeywords:string;
    docDescription:string;
    DataParticipantsFile:string;
    DataScheduleFile:string;
    TargetFormat:TDrawTargetFormat;
    PaperSize:TDrawPaperSize;
    PageOrientation:TDrawPageOrientation;
    Margins:TRect; // margins in millimeters
    DrawFontName:string;
    DrawFontPercent:integer;
    DrawLineWidth:integer;
    DrawInfoLineWidth:integer;

    DrawShowSeed:Boolean;
    DrawSeedChars:string;
    DrawShowSeedBold:Boolean;
    DrawSeededLinesBold:Boolean;

    DataByeInfoText:string;
    DrawGroupInfoText:string;
    DrawGroupLabelText:string;
    DataGroupStats:string;
    DataRandomizeSeeded:boolean;
    DataRandomizeNonSeeded:Boolean;

    DataWinPts:Double;
    DataDrawPts:Double;
    DataLostPts:Double;
    DataPenaltyPts:Double;
    DataHomeAndAway:Boolean;

    DrawParticipantsCount:integer;
    _DrawMeasure:Integer;       // set after reading participants file
    _SeededCount:Integer;       // set after reading participants file and assigning draw measure
    _ExtrasCalculated:Boolean;  // set after PlayerInfo(s).Stats.Extra1 values have been calculated
    _totalRounds:Integer;       // set after generating fixtures
    _lastRound:Integer;         // set after generating fixtures - last scheduled round (played or not)
    DrawTitle:string;
    DrawType:string;
    DrawOtherInfo:string;
    Draw3rdPlace:boolean;

    Adv_Color_InfoBackground:TColor;
    Adv_Color_Line:TColor;
    Adv_Color_InfoLine:TColor;
    Adv_Color_Font:TColor;
    Adv_Color_InfoFont:TColor;
    Adv_Bool_RoundCorners:boolean;

    _FontHeight:integer;
    _SmallFontHeight:integer;
    _XSmallFontHeight:integer;
    _RoundCorners:integer;
  end;

const
  CPointsPerInch = 72;
  CMillimetersPerInch = 25.4;
  LOGPIXELSX = 88;   
  LOGPIXELSY = 90;

  CScheduleFileFields='#MatchID;P1;P2;Winner;Result;Detailed result;P1WinOf;P2WinOf';
  CRankFileFields='#Pos;PID;Plays;Wins;Draws;Losts;Pen.;Score1;Score2;Extra1;Extra2;Points';

var
  CScreenPixelsPerInch:Integer;
  Opt:TCommonOptions;
  Options:TStringList;

var
  Players:TStringList; // Participants list
  Matches:TStringList; // Schedule/Fixtures (matches list)
  Rank:TStringList;    // Standings

  docCreator,
  docCopyright:string;

  DrawDoc:TStringList;

/////////////////////////////////////// MISC ROUTINES /////////////////////////////////
procedure InitSPPI;
var
  DC: HDC;
begin
  DC := GetDC(0);
  try
    CScreenPixelsPerInch := GetDeviceCaps(DC, LOGPIXELSY);
  finally
    ReleaseDC(0,DC);
  end;
end;

function Points2Pixels(APoints: Single): Integer;
begin
  Result := Round(APoints / CPointsPerInch * CScreenPixelsPerInch);
end;

function Millimeters2Points(mm: Double): Single;
begin
  Result := mm / CMillimetersPerInch * CPointsPerInch;
end;

procedure GetPaperMMSize(PaperSize:TDrawPaperSize; PageOrientation:TDrawPageOrientation; var MMWidth:Integer; var MMHeight:Integer);
var
  tmp:Integer;
begin
  case PaperSize of
    dpsA0:begin MMWidth:=841; MMHeight:=1189; end;
    dpsA1:begin MMWidth:=594; MMHeight:=841; end;
    dpsA2:begin MMWidth:=420; MMHeight:=594; end;
    dpsA3:begin MMWidth:=297; MMHeight:=420; end;
    dpsA4:begin MMWidth:=210; MMHeight:=297; end;
    dpsA5:begin MMWidth:=148; MMHeight:=210; end;
    dpsA6:begin MMWidth:=105; MMHeight:=148; end;
    dpsA7:begin MMWidth:=74; MMHeight:=105; end;
    dpsLetter:begin MMWidth:=216; MMHeight:=279; end;
    dpsLegal:begin MMWidth:=216; MMHeight:=356; end;
    dpsJuniorLegal:begin MMWidth:=203; MMHeight:=127; end;
    dpsLedger:begin MMWidth:=432; MMHeight:=279; end;
    dpsTabloid:begin MMWidth:=279; MMHeight:=432; end;
  end;

  if (PaperSize<>dpsUserDefined) and (PageOrientation=dpoLandscape) then
  begin
    tmp:=MMWidth;
    MMWidth:=MMHeight;
    MMHeight:=tmp;
  end;
end;

procedure GetPaperPixelSize(PaperSize:TDrawPaperSize; PageOrientation:TDrawPageOrientation; var Width:Integer; var Height:Integer; MMWidth:Integer; MMHeight:Integer);
begin
  GetPaperMMSize(PaperSize, PageOrientation, MMWidth, MMHeight);
  Width:=Points2Pixels(Millimeters2Points(MMWidth));
  Height:=Points2Pixels(Millimeters2Points(MMHeight));
end;

function StrToPaperSize(StrID:string):TDrawPaperSize;
begin
  StrID:=AnsiUpperCase(StrID); 
  if StrID='A0' then Result:=dpsA0
  else if StrID='A1' then Result:=dpsA1
  else if StrID='A2' then Result:=dpsA2
  else if StrID='A3' then Result:=dpsA3
  else if StrID='A4' then Result:=dpsA4
  else if StrID='A5' then Result:=dpsA5
  else if StrID='A6' then Result:=dpsA6
  else if StrID='A7' then Result:=dpsA7
  else if StrID='LETTER' then Result:=dpsLetter
  else if StrID='LEGAL' then Result:=dpsLegal
  else if StrID='JUNIOR-LEGAL' then Result:=dpsJuniorLegal
  else if StrID='LEDGER' then Result:=dpsLedger
  else if StrID='TABLOID' then Result:=dpsTabloid
  else Result:=dpsA4;
end;

function StrToPageOrientation(StrID:string):TDrawPageOrientation;
begin
  StrID:=AnsiUpperCase(StrID);
  if StrID='LANDSCAPE' then Result:=dpoLandscape
  else Result:=dpoPortrait;
end;

function StrToTargetFormat(StrID:string):TDrawTargetFormat;
begin
  StrID:=AnsiUpperCase(StrID);
  if StrID='PDF' then
    Result:=dtfPdf
  else
  if StrID='EMF-PAGES' then
    Result:=dtfMultiEmf
  else
    Result:=dtfDefault;
end;

{--- ---}

function IsOptionChecked(const v:string; TrueIfNotSet:Boolean):Boolean;
begin
  if TrueIfNotSet and (Length(v)=0) then
    Result:=True
  else
    Result:=(v='1') or ansisametext(v,'on') or ansisametext(v,'true') or ansisametext(v,'yes');
end;

function GetOptionSelectValue(const v:string; const Def:string):string;
var
  p:Integer;
begin
  if Length(v)=0 then
    Result:=Def
  else
  begin
    p:=Pos('|',v);
    if p=0 then
      Result:=v
    else
      Result:=Copy(v,1,p-1);
  end;
end;

function SaveFixturesFile:boolean;
begin
  if Length(Opt.DataScheduleFile)>0 then
    SaveStrToFileEx(CScheduleFileFields+#13#10+Matches.Text,Opt.DataScheduleFile,'utf-8');
  Result:=true;
end;

function GetNormalizedFontHeight(var aPageRect:TRect):Integer;
begin
  if (aPageRect.Right-aPageRect.Left<=0) and (aPageRect.Bottom-aPageRect.Top<=0) then
    Result:=-11
  else
  begin
    Result:=(aPageRect.Bottom-aPageRect.Top) div 96;
    if Result<4 then
      Result:=-4
    else
      Result:=-Result;
  end;
end;

function AddPage(var aPageRect:TRect):TMetafile;
var
  pw,ph:integer;
begin
  Result:=TMetafile.Create;
  Result.Enhanced:=True;
  GetPaperPixelSize(Opt.PaperSize,Opt.PageOrientation, pw, ph, 210, 297);
  Result.Width:=pw;
  Result.Height:=ph;

  aPageRect.Left:=0; aPageRect.Top:=0;
  aPageRect.Right:=Result.Width; aPageRect.Bottom:=Result.Height;
  aPageRect.Left:=aPageRect.Left+Points2Pixels(Millimeters2Points(Opt.Margins.Left));
  aPageRect.Top:=aPageRect.Top+Points2Pixels(Millimeters2Points(Opt.Margins.Top));
  aPageRect.Right:=aPageRect.Right-Points2Pixels(Millimeters2Points(Opt.Margins.Right));
  aPageRect.Bottom:=aPageRect.Bottom-Points2Pixels(Millimeters2Points(Opt.Margins.Bottom));
end;

function GetPageCanvas(const MF:TMetafile):TMetafileCanvas;
begin
  if Length(Opt.docAuthor)+Length(Opt.docDescription)>0 then
    Result:=TMetafileCanvas.CreateWithComment(MF,0, Opt.docAuthor, Opt.docDescription)
  else
    Result:=TMetafileCanvas.Create(MF,0);
end;

function NewPage(var aPageRect:TRect):TMetafileCanvas;
var
  MF:TMetafile;
begin
  MF:=AddPage(aPageRect);
  DrawDoc.AddObject(inttostr(DrawDoc.Count+1),MF);
  Result:=GetPageCanvas(MF);

  Opt._SmallFontHeight:=round(GetNormalizedFontHeight(aPageRect));
  if Opt.DrawFontPercent<>100 then
  begin
    Opt._SmallFontHeight:=round((1.00*Opt.DrawFontPercent/100)*Opt._SmallFontHeight);
    if Opt._SmallFontHeight>-4 then Opt._SmallFontHeight:=-4;
  end;
  Opt._FontHeight:=round(Opt._SmallFontHeight*1.5)+1;
  Opt._XSmallFontHeight:=round(Opt._FontHeight/1.7)+1;
  if Opt._XSmallFontHeight>-4 then Opt._XSmallFontHeight:=-4;

  if Opt.Adv_Bool_RoundCorners then
    Opt._RoundCorners:=trunc(Abs(Opt._FontHeight)) + (trunc(Abs(Opt._FontHeight)) div 4)
  else
    Opt._RoundCorners:=0;

  Result.Font.Name:=Opt.DrawFontName;
  Result.Font.Height:=Opt._FontHeight;
  Result.Font.Color:=Opt.Adv_Color_Font;
end;

function SaveDrawDoc:integer;
var
  i,n:integer;
begin
  if Opt.AlwaysSaveEMF or (Opt.targetFormat=dtfMultiEmf) then
  begin
    n:=Length(IntToStr(DrawDoc.Count));
    if DrawDoc.Count=1 then
      TMetafile(DrawDoc.Objects[0]).SaveToFile(ChangeFileExt(Opt.TargetFile,'.emf'))
    else
      for i:=0 to DrawDoc.count-1 do
        TMetafile(DrawDoc.Objects[i]).SaveToFile(ChangeFileExt(Opt.TargetFile,StringOfChar('0',n-Length(IntToStr(i)))+IntToStr(i)+'.emf'));
  end;
  for i:=0 to DrawDoc.count-1 do
    TMetafile(DrawDoc.Objects[i]).Free;

  if Opt.targetFormat=dtfPdf then
  begin
    if DrawDoc.Count=1 then
      Result:=ConvertEmfToPDFFileA(ChangeFileExt(Opt.TargetFile,'.emf'), Opt.TargetFile, Opt.DrawTitle, Opt.DrawType, Opt.docAuthor, Opt.docKeywords)
    else
      Result:=ConvertEmfToPDFFileA(ChangeFileExt(Opt.TargetFile,'*.emf'), Opt.TargetFile, Opt.DrawTitle, Opt.DrawType, Opt.docAuthor, Opt.docKeywords)
  end
  else
    Result:=0;
end;